"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getBrowser = void 0;
exports.getPage = getPage;
exports.loadPage = loadPage;
exports.fetchMedia = fetchMedia;
exports.fixBaseURI = fixBaseURI;
exports.fetchAsset = fetchAsset;
exports.savePage = savePage;
const tslib_1 = require("tslib");
const node_toolkit_1 = require("@tech_query/node-toolkit");
const fs_extra_1 = require("fs-extra");
const jsdom_1 = require("jsdom");
const lodash_memoize_1 = tslib_1.__importDefault(require("lodash.memoize"));
const path_1 = require("path");
const puppeteer_core_1 = tslib_1.__importDefault(require("puppeteer-core"));
const yaml_1 = require("yaml");
const config_1 = require("./config");
const parser_1 = require("./parser");
exports.getBrowser = (0, lodash_memoize_1.default)(() => puppeteer_core_1.default.launch({
    browser: config_1.isFirefox ? 'firefox' : 'chrome',
    executablePath: config_1.executablePath
}));
async function getPage() {
    const browser = await (0, exports.getBrowser)();
    const page = await browser.newPage();
    await Promise.all([
        page.setRequestInterception(true),
        page.setUserAgent(config_1.userAgent)
    ]);
    page.setDefaultNavigationTimeout(0);
    page.on('request', request => {
        if (['image', 'media'].includes(request.resourceType()))
            request.abort();
        else
            request.continue();
    });
    return page;
}
async function loadPage(URI, root_selector = 'body') {
    if (!config_1.executablePath)
        return jsdom_1.JSDOM.fromURL(URI, {
            userAgent: config_1.userAgent,
            pretendToBeVisual: true,
            runScripts: 'outside-only'
        });
    const page = await getPage();
    await page.goto(URI);
    await page.waitForSelector(`${root_selector}:not(:empty)`);
    const HTML = await page.content();
    await page.close();
    return new jsdom_1.JSDOM(HTML, { url: URI });
}
function fetchMedia(root_1) {
    return tslib_1.__asyncGenerator(this, arguments, function* fetchMedia_1(root, root_path = '') {
        for (const media of root.querySelectorAll(parser_1.MediaSelector)) {
            try {
                var { pathname, protocol, href } = (0, parser_1.sourcePathOf)(media);
            }
            catch (_a) {
                media.remove();
            }
            if (protocol === 'data:')
                var { MIME, data } = yield tslib_1.__await((0, node_toolkit_1.blobFrom)(href));
            else {
                const response = yield tslib_1.__await(fetch(href, {
                    headers: { 'User-Agent': config_1.userAgent }
                }));
                var [MIME] = response.headers.get('Content-Type').split(';'), data = Buffer.from(yield tslib_1.__await(response.arrayBuffer()));
            }
            const name = yield tslib_1.__await((0, parser_1.createFilePath)(data, pathname, root_path));
            media.src = name;
            if (media.style.display === 'none')
                media.style.display = '';
            yield yield tslib_1.__await({ MIME, name, data });
        }
    });
}
function fixBaseURI(document, baseURI) {
    const base = document.createElement('base');
    base.href = baseURI;
    document.head.append(base);
    for (const media of document.querySelectorAll(parser_1.MediaSelector))
        media.setAttribute('src', media.src);
    for (const link of document.querySelectorAll(parser_1.LinkSelector))
        link.setAttribute('href', link.href);
    base.remove();
}
function fetchAsset(document_1, _a, meta_1) {
    return tslib_1.__asyncGenerator(this, arguments, function* fetchAsset_1(document, { scope, rootSelector, baseURI = '', markdown }, meta) {
        var _b, e_1, _c, _d;
        const root = (0, parser_1.parseContent)(document, rootSelector);
        try {
            for (var _e = true, _f = tslib_1.__asyncValues(fetchMedia(root, scope)), _g; _g = yield tslib_1.__await(_f.next()), _b = _g.done, !_b; _e = true) {
                _d = _g.value;
                _e = false;
                const media = _d;
                yield yield tslib_1.__await(media);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (!_e && !_b && (_c = _f.return)) yield tslib_1.__await(_c.call(_f));
            }
            finally { if (e_1) throw e_1.error; }
        }
        if (baseURI)
            fixBaseURI(document, baseURI);
        let markup = `<meta charset="UTF-8">
${root.innerHTML.trim().replace(/(\n\s*)+/g, '\n')}`;
        if (markdown)
            markup = `---
${(0, yaml_1.stringify)(meta)}
---

${parser_1.convertor.turndown(markup)}`;
        yield yield tslib_1.__await({
            MIME: `text/${markdown ? 'markdown' : 'html'}`,
            name: `${(0, parser_1.fileNameOf)(scope)}.${markdown ? 'md' : 'html'}`,
            data: Buffer.from(markup)
        });
    });
}
async function savePage({ source, rootSelector, baseURI = '', markdown, rootFolder = process.cwd() }) {
    var _a, e_2, _b, _c;
    console.time('Fetch');
    const scope = (0, path_1.parse)(source).name, { window: { document } } = await loadPage(source);
    const meta = (0, parser_1.parseMeta)(document, config_1.meta_tag);
    const root_path = (0, path_1.join)(rootFolder, ...(meta.categories || []).map(parser_1.fileNameOf));
    try {
        for (var _d = true, _e = tslib_1.__asyncValues(fetchAsset(document, { scope, rootSelector, baseURI, markdown }, meta)), _f; _f = await _e.next(), _a = _f.done, !_a; _d = true) {
            _c = _f.value;
            _d = false;
            const { name, data } = _c;
            const filePath = (0, path_1.join)(root_path, name);
            await (0, fs_extra_1.outputFile)(filePath, data);
            console.log('[save] ' + filePath);
        }
    }
    catch (e_2_1) { e_2 = { error: e_2_1 }; }
    finally {
        try {
            if (!_d && !_a && (_b = _e.return)) await _b.call(_e);
        }
        finally { if (e_2) throw e_2.error; }
    }
    console.log('--------------------');
    console.timeEnd('Fetch');
}
